@@ -1,5 +1,5 @@
 """Asynchronous cache manager designed for horizontally scaled web applications"""
 
-__version__ = "0.1.0"
+__version__ = "0.5.0"
 
-from .base import Cache as Cache
+from .base import Cache  # noqa
